#!/usr/bin/tclsh

################################################################################
#  1.0  Description
#
#  tcl script for generating OOo toolbar/menu bar icons from kde icon themes
################################################################################

# begin   07/19/03
# author  Rohit Kaul
# email   rohitkaul@sbcglobal.net

# License
# -------
# GPL
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

# Warranty
# --------
# Absolutely none! Use at your own risk!


################################################################################
#  2.0 User Customizations
#
#  Users need to modify only this section
#  (NOTE: keep the toolbar size in OOo to automatic)
################################################################################

 source config.tcl

 if { [file isfile autoConfig.tcl] } {
 	source autoConfig.tcl
    puts "found quickInstall auto-config file"
 }

################################################################################
#  3. Procs
################################################################################

proc guessBgColor { } {
	upvar bg bg

    if { [file exists "~/.qt/qtrc"] != 1} {
    	puts "\nConfig Error:"
    	puts "unable to guess background color"
        puts "set guessBG false (in config.tcl)"
        puts "and set bg <your toolbar color> (in config.tcl)"
        puts "then rerun script"
        exit
    }

	set fp [open "~/.qt/qtrc" r]
	set tmp [read $fp]
	close $fp
	set tmp [split $tmp "\n"]

	set indx [lsearch $tmp "frame*"]

	if { $indx != -1 } {
		set  tmp [lindex $tmp $indx]
		set  tmp [split $tmp "="]
		set bg [lindex $tmp 1]
		puts "Background Color: ($bg)"
	} else {
		puts $tmp
	}
}

proc isValidNum { num } {

	set val -1
	scan $num %d val

    if { $val >= 0 && $val < 1000 } {
    	return 1
    }
		return 0
}


proc isValid { num name } {
	upvar BaseDir BaseDir
	upvar ThemeDir ThemeDir
	global scale_size
	global unscale

	set res [isValidNum $num]

    if { $res == 0 } {
    	return 0;
    } else {

    	set res 0;
		set pName $name
        set filePath $ThemeDir$pName

        if { [file isfile $filePath] } {
        	set res 1
        }

        set pName p$num.png
        set filePath $BaseDir$pName
        if { [file isfile $filePath] } {
        	set res 1
        }

		if { $scale_size != "null" } {
    		if { [lsearch $unscale $num] >= 0 } {
    			set res 0
    		}
		}

		return $res;
    }
}


proc simplifyLine { line splitChar } {

	set  simLine ""
	set  spChar  " "
	set  tmpl   [string trim $line $splitChar]
	set  tmpl   [split $tmpl $splitChar]

	set count 0
	foreach seq $tmpl {
		set  seq  [lindex $tmpl $count]
		set  seq  [string trim $seq]
		set  simLine $simLine$seq$spChar
		incr count
	}
	return $simLine
}


proc genConfigFiles { } {
	upvar OOconfigDir OOconfigDir
    set   tmpl $OOconfigDir
	set   len  [string length $OOconfigDir]
    set   tmpl [string range $tmpl 0 [expr {$len - 13}] ]

	exec rm -Rf $OOconfigDir
    exec mkdir $OOconfigDir
    exec cp -R ./Extras/soffice.cfg $tmpl
}

proc genNavIcons { } {
	upvar OOconfigDir OOconfigDir

	set toolbar "functionbar.xml"
	set toolbar $OOconfigDir$toolbar
    set navSlots {{20907} {20908} {20911} {20909} {20945} {20944} {20958}}


	set fp   [open $toolbar r]
	set lst  [read $fp]
	close $fp

	set lst [split $lst "\n"]
	set indx [lsearch $lst "*</toolbar*"]

	set found 0

	foreach slot $navSlots {
		set slot2 "*slot:$slot*"
		if { [lsearch $lst $slot2] >= 0 } {
			set found 1
			puts "Found: $slot, navigation icons not re-added to toolbar"
			break
		}
	}

	if { $found == 0 } {
		set fp   [open $toolbar w]

		for { set ind 0} {$ind < $indx} {incr ind} {
			puts $fp [lindex $lst $ind]
		}
		puts $fp "<toolbar:toolbarseparator/>"

		foreach slot $navSlots {
			puts $fp "<toolbar:toolbaritem xlink:href=\"slot:$slot\" toolbar:userdefined=\"true\"/>"
		}
		puts $fp "</toolbar:toolbar>"
		close $fp
		puts "Navigation icons added"
	}
}


proc genNavBar { } {
	upvar OOconfigDir OOconfigDir
	exec cp ./Extras/toolbars/userdeftoolbox1.xml $OOconfigDir
}

proc createImageXML { } {
	upvar ImageXml  ImageXml
	upvar SlotFile  SlotFile
	upvar maskImg   maskImg
	upvar BitMapDir BitMapDir
	upvar bg        bg
	upvar ThemeDir  ThemeDir
    upvar BaseDir   BaseDir

	set maskColor $bg;

	set xmlFile $ImageXml
	set nxtFile "nextList.xml"

	# first the header
	set fp    [open $xmlFile  w]
#	set gp    [open $nxtFile w]

	puts $fp "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
	puts $fp "<!DOCTYPE image:imagecontainer PUBLIC \"-//OpenOffice.org//DTD OfficeDocument 1.0//EN\" \"image.dtd\">"
	puts $fp "<image:imagescontainer xmlns:image=\"http://openoffice.org/2001/image\""
	puts $fp "xmlns:xlink=\"http://www.w3.org/1999/xlink\">"

	if { $maskImg == "true"} {
	puts $fp "<image:images xlink:type=\"simple\" xlink:href=\"$BitMapDir/userimages.bmp\" image:maskmode=\"maskbitmap\" image:maskurl=\"$BitMapDir/userimagesmask.bmp\">"
	} else {
	puts $fp "<image:images xlink:type=\"simple\" xlink:href=\"$BitMapDir/userimages.bmp\" image:maskmode=\"maskcolor\" image:maskcolor=\"$maskColor\">"
	}


	set op [open $SlotFile r]
	set lst [read $op]
	close $op

	set inc 0
	set lst [split $lst "\n"]

	foreach icon $lst {
	set icon  [simplifyLine $icon ","]
	set icon  [split $icon]

	set pNum  [lindex $icon 0]
	set pName [lindex $icon 1]
	set pSlot [lindex $icon 2]

      if { [isValid $pNum $pName] } {
		puts $fp "<image:entry image:bitmap-index=\"$inc\" image:command=\"$pSlot\"/>"

		incr inc
		}
	}
	puts $fp "</image:images>"
    puts $fp "</image:imagescontainer>"

	close $fp
}

proc nextImg {inc pNum pName delx} {
upvar x0    x0
upvar y0    y0
upvar out1  out1
upvar out2  out2
upvar cx    cx
upvar cy    cy
upvar dx    dx
upvar dy    dy
upvar bg    bg
upvar BitMapDir  BitMapDir
upvar BaseDir    BaseDir
upvar ThemeDir   ThemeDir
upvar themeIcons themeIcons
global maskImg
global scale_size
global scale_sharp
global unscale

	set pNum0  $pNum
	set pNum   "p$pNum"
	set pNumfl "$pNum.png"
	set img    "$BaseDir$pNumfl"

	set custom 0

	if { [lsearch $themeIcons $pName] != -1 } {
		set img $ThemeDir$pName
		set custom 1
	}

	if { [lsearch $themeIcons $pNumfl] != -1 } {
		set img $ThemeDir$pNumfl
		set custom 1
	}

    if { $scale_size != "null" } {

    	exec composite -resize $scale_size -sharpen  $scale_sharp \
                       -geometry +$x0+$y0 $img $out1 $out1
        if { $maskImg == "true" } {
		set imgMsk $img
		exec composite -resize $scale_size -sharpen $scale_sharp \
                       -geometry +$x0+$y0 $imgMsk $out2 $out2
    	}
    } else {
		exec composite -geometry +$x0+$y0 $img $out1 $out1

        if { $maskImg == "true" } {
		set imgMsk $img
		exec composite -geometry +$x0+$y0 $imgMsk $out2 $out2
    	}

    }


	if { $custom == 0 } {
		puts -nonewline "."
	} else {
		puts -nonewline "*"
	}

	flush stdout
}


proc copyKDE_Icons { } {
	upvar ICN_DIR   ICN_DIR
    upvar LCN_DIR   LCN_DIR
	upvar KDE_GLOB  KDE_GLOB
	upvar homeDir   homeDir
    upvar ThemeDir  ThemeDir
	upvar kde_icon_sz kde_icon_sz

    set x "x"
    set isz $kde_icon_sz$x$kde_icon_sz

    set search \\\[Icons\\\]

	puts "home dir = $homeDir"
    set Kglobals $homeDir/$KDE_GLOB
    puts "kde config file: $Kglobals"

    set kp [open $Kglobals r]
	set tmp [read $kp]
	close $kp

	set tmp [split $tmp "\n"]
	set indx [lsearch -regexp $tmp $search]

	if { $indx != -1 } {
    	incr indx
		set  tmp [lindex $tmp $indx]
		set  tmp [split $tmp "="]
		set  kicon [lindex $tmp 1]
		puts "kde icon set: ($kicon)"

        set  kdir $ICN_DIR$kicon/
		set  ldir $homeDir/$LCN_DIR$kicon/
        set  isGlobal [file isdirectory $kdir]
        set  isLocal  [file isdirectory $ldir]

        set iconPath $kdir$isz
        if { $isGlobal == 1} {
        	puts "found global icons: $kdir"
        } elseif { $isLocal == 1} {
        	puts "found local icons: $ldir"
			set  iconPath $ldir$isz
        } else {
			puts "kde icon dir not found, using defaults"
            return
        }
        exec rm -Rf "kde_theme"
        set  kTheme "kde_theme/$isz/"
        exec mkdir -p $kTheme

        set acts "actions"
        set apps "apps"


        puts -nonewline "copying kde icons ... "
        # set files [lsort [glob {$iconPath/$acts/*.png}]]
		exec ls -w 1 $iconPath/$acts/ > acts.txt
		exec ls -w 1 $iconPath/$apps/ > apps.txt

		set fp [open acts.txt r]
        set iconLst [read $fp]
		close $fp
		set iconLst [split $iconLst "\n"]
		foreach icon $iconLst {
			if { [string length $icon] > 3 } {
			exec cp -f $iconPath/$acts/$icon $kTheme
            }
        }

        set fp [open apps.txt r]
        set iconLst [read $fp]
		close $fp
		set iconLst [split $iconLst "\n"]
		foreach icon $iconLst {
			if { [string length $icon] > 3 } {
			exec cp -Rf $iconPath/$apps/$icon $kTheme
            }
        }
		puts " done"
        set ThemeDir $kTheme

	} else {
		puts "Kde icon theme not found, using default"
	}
}


proc cleanup { } {
	exec rm -f lst.txt
    exec rm -f acts.txt
    exec rm -f apps.txt
	exec rm -f nextList.xml
    exec rm -f imagelist.xml
}


################################################################################
# 4. Initialize
################################################################################
set SlotFile   "slotlist.csv"
set ImageXml   "imagelist.xml"
set BitMapDir  "genBitmaps"
set numIcons   0

	if { $useKDEdefault == "true" } {
		copyKDE_Icons
	}

    puts "ThemeDir: $ThemeDir"
    set sz [split $ThemeDir "/"]
	set sz  [lindex $sz [expr {[llength $sz] -2}]]

    set cx 0
    set cy 0

	scan $sz %d%c%d cx xx cy

	puts "icon-size = ($cx,$cy)"
	set BaseDir     ".BaseIcons/22x22/"


    if { $cy < 19 } {
    set BaseDir     ".BaseIcons/16x16/"
    }

	set op [open $SlotFile r]
	set lst [read $op]
	close $op

	set lst [split $lst "\n"]
	foreach icon $lst {
    	set icon  [simplifyLine $icon ","]
		set  pNum  [lindex $icon 0];
        set  pName [lindex $icon 1];
        if { [isValid $pNum $pName] } {
			incr numIcons
		}
	}

	puts "num icons: $numIcons"

set out1   "$BitMapDir/userimages.bmp"
set out2   "$BitMapDir/userimagesmask.bmp"
set OObitmapDir $OOconfigDir$BitMapDir


	exec ls -w 1 $ThemeDir > lst.txt
	set fp [open lst.txt r]
	set themeIcons [read $fp]
	close $fp
	set themeIcons [split $themeIcons "\n"]


    if { $scale_size != "null" } {
		scan $scale_size %d%c%d cx xx cy
    }

	set cx [expr {$cx + 2*$dx}]
	set cy [expr {$cy + 2*$dy}]

	set x0 $dx
	set y0 $dy

	set len [expr {$numIcons*$cx}]

	set count 0
	set fnbar "functionbar.xml"

    puts "OpenOffice config directory:"
    puts $OOconfigDir
	set confRes [file isdirectory $OOconfigDir]
    if { $genConfig == "true" } {
		if { $confRes == 0 } {
			puts "config directory not found, generating"
            genConfigFiles
        }
		set res [file isfile $OOconfigDir$fnbar]
        if { $res == 0 } {
			puts "functionbar.xml not found, generating"
            genConfigFiles
        }
    } else {
		if {$confRes == 0} {
        	puts "config directory not found, creating"
        	exec mkdir $OOconfigDir
        }
    }

 	while { $count <= 100 } {
			set bakDir $OOconfigDir
			set bakDir [string trimright $bakDir "/"]
			set bakDir "$bakDir.bak$count/"

		set res [file isdirectory $bakDir]
		if { $res == 0 } {
			puts "Backing up config dir (soffice.cfg) to:"
			puts $bakDir
			exec cp -R $OOconfigDir $bakDir
			break;
		}
		incr count

		if {$count==100} {
			puts "Too many backup directories...(soffice.cfg.bak*)!"
			puts "exiting"
			exit
		}
	}

################################################################################
#  5. Generate tool-bar
################################################################################

	if { $guessBG == "true"} {
	guessBgColor
	}

	createImageXML

	if { $addNavIcons == "true"} {
    genNavIcons
    }

	if { $addNavBar == "true"} {
    genNavBar
    }

	exec rm -Rf $BitMapDir
	exec mkdir  $BitMapDir

	set x "x"
	exec convert -size $len$x$cy xc:$bg    $out1
    if { $maskImg == "true" } {
	exec convert -size $len$x$cy xc:white  $out2
    }

	# start iterations
	set op [open $SlotFile r]
	set lst [read $op]
	close $op

	set inc 0
	set lst [split $lst "\n"]



	foreach icon $lst {
		set icon  [simplifyLine $icon ","]
		set icon  [split $icon]

		set pNum  [lindex $icon 0]
		set pName [lindex $icon 1]
		set pSlot [lindex $icon 2]


        if { [isValid $pNum $pName] } {
			nextImg $inc $pNum $pName $cx
			incr inc 1
			set x0 [expr {$x0 + $cx}]
		}
	}

	puts " done"

	exec rm -Rf OObitmapDir
	exec cp -R $BitMapDir $OOconfigDir
	exec cp -f $ImageXml $OOconfigDir

	cleanup

